clear all
set more off


* GENERAL NOTE: many variables in the public use files are stored such
* that the values themselves are not of interest, but the value labels are
* (e.g., the "birthyear" variable codes being born in 1977 as "1", but the
* corresponding value label is the more informative "1977"). The code
* below fixes this by overwriting the values with the codes stored in the
* value labels for all variables used in the analysis.


**************************************************************************
**************************************************************************
** 					  LOAD PUBLIC USE STUDENT DATA
**************************************************************************
**************************************************************************

use STAR_students_public_use_for_JEEA.dta


**************************************************************************
**************************************************************************
**            CORRECT CODING ERRORS IN CONTINUOUS VARIABLES
** 		 (replace values with those stored in the value labels)
**************************************************************************
**************************************************************************

* grade K-3 SAT scores
foreach g in k 1 2 3 {
	foreach v in g`g'tmathss g`g'treadss g`g'tlistss g`g'wordskillss {
		decode `v', gen(`v'_decoded)
		drop `v'
		gen `v' = real(`v'_decoded)
		drop `v'_decoded
	}
}

* teacher experience and class size
foreach v in gktyears g1tyears g2tyears g3tyears gkclasssize g1classsize g2classsize g3classsize /*g4nclass*/ {
	decode `v', gen(`v'_decoded)
	drop `v'
	gen `v' = real(`v'_decoded)
	drop `v'_decoded
}

* non-cognitive skills
foreach v in g4pteffr g4ptinit g4ptnonp g4ptvalu g8pmeffr g8pminit g8pmnonp g8peeffr g8peinit g8penonp {
	decode `v', gen(`v'_decoded)
	drop `v'
	gen `v' = real(`v'_decoded)
	drop `v'_decoded
}


**************************************************************************
**************************************************************************
**              PREPARE BACKGROUND AND CONTROL VARIABLES
**************************************************************************
**************************************************************************

**************************************************************************
*		indicators for when students entered and left the experiment
**************************************************************************

* indicators: in STAR sample in grades K, 1, 2, 3?
recode X (1 = 0) (2 = 1), gen(gk)
recode X_1 (1 = 0) (2 = 1), gen(g1)
recode X_2 (1 = 0) (2 = 1), gen(g2)
recode X_3 (1 = 0) (2 = 1), gen(g3)
gen g4 = (g4tchid != .)

* define "entrants"; note: these can include students that were not 
* observed in the previous year but who were observed two years ago
gen gkentrant = (gk == 1)
gen g1entrant = ((gk == 0) * (g1 == 1))
gen g2entrant = ((gk == 0) * (g1 == 0) * (g2 == 1))
gen g3entrant = ((gk == 0) * (g1 == 0) * (g2 == 0) * (g3 == 1))


**************************************************************************
*                  student demographic characteristics
**************************************************************************

* indicators for male, black, and free lunch
recode gender (2 = 0) (1 = 1), gen(male)
recode race (1 3 4 5 6 = 0) (2 = 1) , gen(black)
recode gkfreelunch (2 = 0)
recode g1freelunch (2 = 0)
recode g2freelunch (2 = 0)
recode g3freelunch (2 = 0)
label drop gkfreelunch g1freelunch g2freelunch g3freelunch

* age on September 30, 1985 (TN cutoff for kindergarten entry)
recode birthyear (1 = 1977) (2 = 1978) (3 = 1979) (4 = 1980) (5 = 1981)
gen dateofbirth = mdy(birthmonth, birthday, birthyear)
format dateofbirth %d
gen age_days = mdy(09,30,1985) - dateofbirth
gen age_years = age_days / 365.25

* indicator for being old for grade
gen born_before_cutoff = (month(dateofbirth) <= 09) if dateofbirth < .
gen gkexpgrade = 1985 - year(dateofbirth) - 6 + born_before_cutoff if gk == 1
gen gkoldforgrade = (gkexpgrade > 0) if gkexpgrade != .
gen g1expgrade = 1986 - year(dateofbirth) - 6 + born_before_cutoff if g1 == 1
gen g1oldforgrade = (g1expgrade > 1) if g1expgrade != .
gen g2expgrade = 1987 - year(dateofbirth) - 6 + born_before_cutoff if g2 == 1
gen g2oldforgrade = (g2expgrade > 2) if g2expgrade != .
gen g3expgrade = 1988 - year(dateofbirth) - 6 + born_before_cutoff if g3 == 1
gen g3oldforgrade = (g3expgrade > 3) if g3expgrade != . 

* 'expected age' at kindergarten entry
gen exp_age_years = age_years - gkexpgrade
drop born_before_cutoff g*expgrade


**************************************************************************
*                          identify repeaters
**************************************************************************

* kindergarten repeater
recode gkrepeat (1 = 0) (2 = 1), gen(gkrepeater)
drop gkrepeat

* old-for-grade kindergarten repeater
gen gkofgrepeater = 0
replace gkofgrepeater = 1 if gkrepeater == 1 & gkoldforgrade == 1


**************************************************************************
*             create school, teacher, and class identifiers
**************************************************************************

* school and teacher ids (create as strings)
decode gkschid, gen(gkschoolid)
decode g1schid, gen(g1schoolid)
decode g2schid, gen(g2schoolid)
decode g3schid, gen(g3schoolid)
decode g4schid, gen(g4schoolid)
decode g5schid, gen(g5schoolid)
decode g6schid, gen(g6schoolid)
decode g7schid, gen(g7schoolid)
decode g8schid, gen(g8schoolid)
decode gktchid, gen(gkteachid)
decode g1tchid, gen(g1teachid)
decode g2tchid, gen(g2teachid)
decode g3tchid, gen(g3teachid)
decode g4tchid, gen(g4teachid)

* class id (from teacher id) and class type
egen gkclassid = group(gkschoolid gkteachid)
egen g1classid = group(g1schoolid g1teachid)
egen g2classid = group(g2schoolid g2teachid)
egen g3classid = group(g3schoolid g3teachid)
egen g4classid = group(g4schoolid g4teachid)
recode gkclasstype (2 3 = 0), gen(gksmallclass)
recode g1classtype (2 3 = 0), gen(g1smallclass)
recode g2classtype (2 3 = 0), gen(g2smallclass)
recode g3classtype (2 3 = 0), gen(g3smallclass)

* class and school tags (repeaters are never tagged)
foreach g in k 1 2 3 4 {
	
	sort g`g'classid gkrepeater
	egen g`g'classtag = tag(g`g'classid)
	
	sort g`g'schoolid gkrepeater
	egen g`g'schooltag=tag(g`g'schoolid)
}


**************************************************************************
*               	    teacher characteristics
**************************************************************************

* race and education (note: no need to clean experience variable)
foreach g in k 1 2 3 {
	
	recode g`g'trace (1 3 = 0) (2 = 1), gen(g`g'teachblack)
	label define g`g'teachblack 0 "white" 1 "black"
	label values g`g'teachblack g`g'teachblack
	label variable g`g'teachblack "grade `g' teacher is black"

	recode g`g'thighdegree (2 = 0) (3 4 5 6 = 1), gen(g`g'teachadvdeg)
	label define g`g'teachadvdeg 0 "BA degree" 1 "post-BA degree"
	label values g`g'teachadvdeg g`g'teachadvdeg
	label variable g`g'teachadvdeg "grade `g' teacher has post-BA degree"
}


**************************************************************************
*                           class composition
**************************************************************************

* define peer characteristics (class-level leave-me-out-means); note: re-
* peaters are ignored (do not want them to "contaminate" peer variables)
foreach g in k 1 2 3 {
	foreach c in male black g`g'freelunch age_years g`g'oldforgrade {
		bysort g`g'classid: egen g`g'classmn_`c' = mean(`c') if g`g'classid != . & ((gk == 1 & gkrepeater == 0) | gk == 0)
		bysort g`g'classid: egen g`g'class_obs_`c' = total(`c' != .) if g`g'classid != . & ((gk == 1 & gkrepeater == 0) | gk == 0)

		gen g`g'classlom_`c' = (g`g'classmn_`c' * g`g'class_obs_`c' - `c') / (g`g'class_obs_`c' - 1) if `c' != . & ((gk == 1 & gkrepeater == 0) | gk == 0)
		replace g`g'classlom_`c' = (g`g'classmn_`c' * g`g'class_obs_`c') /(g`g'class_obs_`c') if `c' == . & ((gk == 1 & gkrepeater == 0) | gk == 0)
	}
}

* relative age in kindergarten
gen gkclass_relative_age = age_years - gkclasslom_age_years


**************************************************************************
*                        school characteristics
**************************************************************************

* student body characteristics
bysort gkschoolid: egen gkschool_nstudents = total(gkschoolid != "")
bysort gkschoolid: egen gkschool_shmale = mean(male)
bysort gkschoolid: egen gkschool_shblack = mean(black)
bysort gkschoolid: egen gkschool_shfreelunch = mean(gkfreelunch)
bysort gkschoolid: egen gkschool_mnage = mean(age_years)

* school location
tab gksurban, gen(gkschool_urban)

* number and share of repeaters
bysort gkschoolid: egen gkschool_nofgrep = total(gkofgrepeater)
gen gkschool_shofgrep = 100 * (gkschool_nofgrep / gkschool_nstudents)


**************************************************************************
**************************************************************************
**                      PREPARE TREATMENT VARIABLES
**************************************************************************
**************************************************************************

**************************************************************************
*                              KG treatments
**************************************************************************

* number of repeaters in class
bysort gkclassid: egen gkclass_nrep = total(gkrepeater == 1) if gk == 1
gen gkclass_rep_dummy = (gkclass_nrep > 0) if gk == 1 

* number of old-for-grade repeaters in class
bysort gkclassid: egen gkclass_nofgrep = total(gkofgrepeater == 1) if gk == 1
gen gkclass_ofgrep_dummy = (gkclass_nofgrep > 0) if gk == 1

* share of old-for-grade repeaters in class
gen gkclass_shofgrep = gkclass_nofgrep / gkclasssize


**************************************************************************
*                   repeaters in class in later grades?
**************************************************************************

foreach g in 1 2 3 4 {
	bysort g`g'classid: egen g`g'class_nofgrep = total(gkofgrepeater == 1) if g`g' == 1
	gen g`g'class_ofgrep_dummy = (g`g'class_nofgrep > 0) if g`g' == 1
}

/*bysort g2classid: egen g2class_nofgrep=total(gkofgrepeater==1) if g2==1
gen g2class_ofgrep_dummy=(g2class_nofgrep>0) if g2==1
bysort g3classid: egen g3class_nofgrep=total(gkofgrepeater==1) if g3==1
gen g3class_ofgrep_dummy=(g3class_nofgrep>0) if g3==1
bysort g4classid: egen g4class_nofgrep=total(gkofgrepeater==1) if g4==1
gen g4class_ofgrep_dummy=(g4class_nofgrep>0) if g4==1*/


**************************************************************************
**************************************************************************
**                      PREPARE OUTCOME VARIABLES
**************************************************************************
**************************************************************************

**************************************************************************
*                           sample selection
**************************************************************************

* drop 28 kindergarten students with missing information on repetition
drop if gkentrant == 1 & gkrepeater == .

* drop 5 non-repeating kindergarten students with missing demographics
drop if gk == 1 & (male == . | black == . | gkfreelunch == . | age_years == . | gkoldforgrade == .)

* a single student has g3 == 1 but g3classid == .; correct this
replace g3 = 0 if g3classid == .


**************************************************************************
*                             KG SAT scores
**************************************************************************

* standardize test scores
qui sum gktmathss if gkrepeater == 0
gen gkmath_std = (gktmathss - r(mean)) / sqrt(r(Var))
qui sum gktreadss if gkrepeater == 0
gen gkread_std = (gktreadss - r(mean)) / sqrt(r(Var))


**************************************************************************
*                later outcomes based on public use data
**************************************************************************

* standardize SAT scores in grades 1-3 
foreach g in 1 2 3 {
	qui sum g`g'tmathss if gk == 1 & gkrepeater == 0
	gen g`g'math_std = (g`g'tmathss - r(mean)) / sqrt(r(Var))
	qui sum g`g'treadss if gk == 1 & gkrepeater == 0
	gen g`g'read_std = (g`g'treadss - r(mean)) / sqrt(r(Var))
}

* grade 4 non-cognitive skills
gen g4ptdpln = 24 - g4ptnonp // this reverts the "non-participation" or "discipline" index
gen g4ptappr = g4ptvalu
foreach s in g4pteffr g4ptinit g4ptdpln g4ptappr {
	qui sum `s' if gk == 1 & gkrepeater == 0
	gen `s'_std = (`s' - r(mean)) / sqrt(r(Var))
}

* grade 8 non-cognitive skills
gen g8pmnonp_inv = 18 - g8pmnonp
gen g8penonp_inv = 18 - g8penonp
gen g8pteffr = (g8pmeffr + g8peeffr) / 2 // checked: either missing for both or missing for none
gen g8ptinit = (g8pminit + g8peinit) / 2 // checked: either missing for both or missing for none
gen g8ptdpln = (g8pmnonp_inv + g8penonp_inv) / 2 // checked: either missing for both or missing for none
gen g8pmappr = 12 - g8pmcrts - g8pmcrit // checked: either missing for both or missing for none
gen g8peappr = 12 - g8pecrts - g8pecrit // checked: either missing for both or missing for none
gen g8ptappr = (g8pmappr + g8peappr) / 2
foreach s in g8pteffr g8ptinit g8ptdpln g8ptappr {
	qui sum `s' if gk == 1 & gkrepeater == 0
	gen `s'_std = (`s' - r(mean)) / sqrt(r(Var))
}

* non-cognitive skills index
egen g4g8noncog = rowmean(g4pteffr_std g4ptinit_std g4ptdpln_std g4ptappr_std g8pteffr_std g8ptinit_std g8ptdpln_std g8ptappr_std)
qui sum g4g8noncog if gk == 1 & gkrepeater == 0
gen g4g8noncog_std = (g4g8noncog - r(mean)) / sqrt(r(Var))

* high school GPA: impute missing with mean GPA across available subjects
gen hsgpa = hsgpaoverall
egen aux = rowmean(hsgpamath hsgpascience hsgpaflang)
replace hsgpa = aux if hsgpa == .
drop aux

* high school graduation and college test-taking
recode hsgrdadd (1 4 = 0) (2 3 5 = 1), gen(hsgraduate)
recode hstest (1 = 0) (2 = 1) (. = 0)
label drop hstest

foreach v in hsgraduate hsgpa hstest {
	qui sum `v' if gkrepeater==0
	gen `v'_std = (`v' - r(mean)) / (sqrt(r(Var)))
}

* long-term index
egen hsindex = rowmean(hsgraduate_std hsgpa_std hstest_std)
qui sum hsindex if gkrepeater == 0
gen hsindex_std = (hsindex - r(mean)) / (sqrt(r(Var)))


**************************************************************************
*                    CTBS test scores and grade level
**************************************************************************

* year-specific test scores
forvalues y = 90(1)97 {
	qui sum math`y' if gk == 1 & gkrepeater == 0
	gen math`y'_std = (math`y' - r(mean)) / sqrt(r(Var))
	qui sum read`y' if gk == 1 & gkrepeater == 0
	gen read`y'_std = (read`y' - r(mean)) / sqrt(r(Var))
}

* below grade in 1994?
gen belowgrade94 = 0 if grade94 != .
replace belowgrade94 = 1 if belowgrade94 == 0 & grade94 < 8


**************************************************************************
**************************************************************************
**                   ASSIGN VARIABLE LABELS & SAVE DATA                 **
**************************************************************************
**************************************************************************

* kindergarten repeaters
label variable gkrepeater "KG repeater"
label define gkrepeater 0 "non-repeater" 1 "repeater"
label values gkrepeater gkrepeater
label variable gkofgrepeater "Old-for-grade KG repeater"
label define gkofgrepeater 0 "non-repeater" 1 "ofg-repeater"
label values gkofgrepeater gkofgrepeater

* student demographics
label variable male "Male"
label variable black "Black"
label variable age_years "Age in years"
label variable gkoldforgrade "KG old for grade"
label variable gkfreelunch "KG free lunch"

* class type and class size
label variable gkclasssize "KG class size"
label variable gksmallclass "KG small class"

* school characteristics
label variable gkschool_nstudents "KG school enrollment"
label variable gkschool_shmale "KG school perc female"
label variable gkschool_shblack "KG school perc black"
label variable gkschool_shfreelunch "KG school perc free lunch"

* measures of repeater exposure
label variable gkclass_rep_dummy "Repeater in class"
label variable gkclass_ofgrep_dummy "Ofg repeater in KG class"

saveold STAR_students_clean.dta, replace
